using System;
using System.Collections;

namespace _13_CardCollection
{
    public enum CardSuits
    {
        Hearts = 1, Spades, Diamonds, Clubs
    }
    public enum CardValues
    {
        Two=2, Three, Four, Five, Six, Seven, Eight, Nine, Ten,
        Jack = 11, Queen, King, Ace
    }

    public class Card : IComparer
    {
        int number;
        CardSuits suit;

        public Card() : this( CardValues.Two, CardSuits.Hearts )
        {
        }

        public Card( CardValues cvalue, CardSuits csuit )
        {
            if( (int) cvalue > 1 && (int)cvalue < 15 )
                number = (int) cvalue;
            else
                number = 2;


            if( (int) csuit > 0 && (int) csuit < 5 )
                suit = csuit;
            else
                suit = CardSuits.Hearts;
        }

        public override string ToString()
        {
            string ret;

            if( number > 10 )
            {
                ret = Enum.GetName( typeof(CardValues), number );
            }
            else
            {
                ret = number.ToString();
            }

            ret += " of ";

            ret += Enum.GetName( typeof(CardSuits), suit );

            return ret;
        }

        public CardSuits CardSuit
        {
            get
            {
                return suit;
            }
            set
            {
                if( (int) value > 0 && (int) value < 5 )
                    suit = value;
            }
        }

        public CardValues CardValue
        {
            get
            {
                return (CardValues) number;
            }
            set
            {
                if( (int) value > 1 && (int) value < 15 )
                {
                    number = (int) value;
                }
            }
        }
 
        public int Compare(object x, object y)
        {
            Card a = (Card) x;
            Card b = (Card) y;

            if( (int) a.CardSuit < (int) b.CardSuit )
                return -1;
            if( (int) a.CardSuit > (int) b.CardSuit )
                return 1;

            if( (int) a.CardValue < (int) b.CardValue )
                return -1;
            if( (int) a.CardValue > (int) b.CardValue )
                return 1;

            return 0;
        }
    }

	public class CardHand : CollectionBase
	{
		public CardHand()
		{
        }

        public override string ToString()
        {
            string ret = "";

            foreach( Card c in base.InnerList )
                ret += c.ToString() + "\n";

            return ret;
        }

        public void GenerateRandomHand( int numCards )
        {
            Random rand = new Random( (int)DateTime.Now.Ticks );

            // copy to array and sort
            ArrayList sortedCards = (ArrayList) base.InnerList.Clone();
            sortedCards.Sort( new Card() );

            for( int i = 0; i < numCards; i++ )
            {
                Card newCard;

                // make sure card doesn't already exist
                do
                {
                    int val = rand.Next( 2, 15 );
                    int suit = rand.Next( 1, 4 );

                    newCard = new Card( (CardValues) val, (CardSuits) suit );
                } while( sortedCards.BinarySearch(newCard, newCard ) > 0);

                sortedCards.Add( newCard );
                Add( newCard );
                sortedCards.Sort( new Card() );
            }
        }

        public int Add( Card value )
        {
            return base.InnerList.Add( (object)value );
        }
 
        public void Remove( Card value )
        {
            base.InnerList.Remove( value );
        }

        public void Sort()
        {
            base.InnerList.Sort( new Card() );
        }
    }
}
