using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace _6_CustomToolbox
{
    [
    DefaultProperty("Percentage"),
    ToolboxBitmap(typeof(TemperatureGauge4), "ToolboxIcon.bmp")
    ]
	public class TemperatureGauge4 : System.Windows.Forms.Control
	{
		private System.ComponentModel.Container components = null;
        private int percent = 50;
        private Color gaugeColor = Color.Red;
        private string fileName = "";

		public TemperatureGauge4()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		private void InitializeComponent()
		{

        }
		#endregion

		protected override void OnPaint(PaintEventArgs pe)
		{
            // compute rects
            Rectangle gaugeRect = new Rectangle( this.Width/6, 0, this.Width-(2*this.Width/6), this.Height-(this.Height/10) );
            Rectangle bulbRect = new Rectangle(0, this.Height-(this.Height/5), this.Width-2, this.Height/5-2);
            Rectangle emptyRect = new Rectangle( gaugeRect.X, gaugeRect.Y, gaugeRect.Width, (gaugeRect.Height/100)*(100-percent));
            Rectangle filledRect = new Rectangle( gaugeRect.X, emptyRect.Y+emptyRect.Height, gaugeRect.Width, gaugeRect.Height-emptyRect.Height );
           
            // draw unfilled rect
            pe.Graphics.DrawRectangle( new Pen(new SolidBrush( gaugeColor )), emptyRect );

            // draw filled portion
            pe.Graphics.FillRectangle( new SolidBrush( gaugeColor ), filledRect );

            // draw bulb
            pe.Graphics.FillEllipse( new SolidBrush( gaugeColor ), bulbRect );
            

            // draw ticks
            for( int i = 1; i < 10; i ++ )
            {
                pe.Graphics.FillEllipse( new SolidBrush( Color.Black ), gaugeRect.X, (gaugeRect.Height/100)*(10*i), gaugeRect.Width/3, 3 );
            }

			base.OnPaint(pe);
		}

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged (e);
            Invalidate();
        }

        [Description("Percentage of gauge that is filled."),Category("Data"),
        ParenthesizePropertyNameAttribute(true), DefaultValue(100)]
        public int Percentage
        {
            get
            {
                return percent;
            }
            set
            {
                if( value < 0 || value > 100 )
                    return;
                percent = value;

                Invalidate();
            }
        }

        [Description("Color of filled gauge."),Category("Appearance"), 
        ParenthesizePropertyNameAttribute(true), DefaultValue(typeof(Color), "Red")]
        public Color GaugeColor
        {
            get
            {
                return gaugeColor;
            }
            set
            {
                gaugeColor = value;
                Invalidate();
            }
        }

        [Description("Filename to log runtime data to."),
        Category("Behaviour"), 
        ParenthesizePropertyNameAttribute(true), 
        EditorAttribute(typeof(System.Windows.Forms.Design.FileNameEditor), typeof(System.Drawing.Design.UITypeEditor))
        ]
        public string LogFile
        {
            get
            {
                return fileName;
            }
            set
            {
                fileName = value;
            }
        }
	}
}
