using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace _7_ExtenderProviders
{
    [
    ProvideProperty("BalloonText",typeof(Control))
    ]
    public class BalloonTip : Component, System.ComponentModel.IExtenderProvider
    {
		private System.ComponentModel.Container components = null;
        private Hashtable balloonTexts;
        private System.Timers.Timer hoverTimer;
        private System.Timers.Timer tipTimer;
        private System.Windows.Forms.Control activeControl;
        private Point activeControlCursor = new Point(0);
        private BalloonForm balloonForm = null;

		public BalloonTip()
		{
			InitializeComponent();
            balloonTexts = new Hashtable();
            balloonForm = new BalloonForm();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

        bool IExtenderProvider.CanExtend(object target) 
        {
            if (target is Control && !(target is BalloonTip)) 
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        [DefaultValue("")]
        public string GetBalloonText(Control control) 
        {
            string balloonText = (string)balloonTexts[control];
            if (balloonText == null) 
            {
                balloonText = string.Empty;
            }
            return balloonText;
        }

        public void SetBalloonText(Control control, string balloonText) 
        {
            if (balloonText == null) 
            {
                balloonText = string.Empty;
            }

            if (balloonText.Length == 0) 
            {
                balloonTexts.Remove(control);

                control.MouseHover -= new EventHandler(OnControlMouseHover);
                control.MouseMove -= new MouseEventHandler(OnControlMouseMove);
            }
            else 
            {
                balloonTexts[control] = balloonText;

                control.MouseHover += new EventHandler(OnControlMouseHover);
                control.MouseMove += new MouseEventHandler(OnControlMouseMove);
            }
        }

        private void OnControlMouseHover(object sender, EventArgs e) 
        {
            if( activeControl == (Control) sender )
                return;

            activeControl = (Control)sender;
            hoverTimer.Start();
        }
        private void OnControlMouseMove( object sender, MouseEventArgs e )
        {
            activeControlCursor.X = e.X;
            activeControlCursor.Y = e.Y;
        }

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.hoverTimer = new System.Timers.Timer();
            this.tipTimer = new System.Timers.Timer();
            ((System.ComponentModel.ISupportInitialize)(this.hoverTimer)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.tipTimer)).BeginInit();
            // 
            // hoverTimer
            // 
            this.hoverTimer.Elapsed += new System.Timers.ElapsedEventHandler(this.hoverTimer_Elapsed);
            // 
            // tipTimer
            // 
            this.tipTimer.Interval = 1000;
            this.tipTimer.Elapsed += new System.Timers.ElapsedEventHandler(this.tipTimer_Elapsed);
            ((System.ComponentModel.ISupportInitialize)(this.hoverTimer)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.tipTimer)).EndInit();

        }
		#endregion

        private void hoverTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            hoverTimer.Stop();

            // set balloon text
            balloonForm.Text = balloonTexts[activeControl].ToString();

            tipTimer.Start();
            
            balloonForm.Location = activeControl.PointToScreen(activeControlCursor);

            balloonForm.ShowDialog();
        }

        private void tipTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            tipTimer.Stop();

            if( balloonForm != null )
            {
                balloonForm.Hide();
            }
        }
	}
}
