using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Drawing.Design;

namespace _8_UITypeEditor
{
    
    public class TemperatureGaugeDesigner : UITypeEditor
    {
        private IWindowsFormsEditorService editorService = null;
        private TemperatureGauge5 tempGauge = null;

        public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            
            if (context != null && context.Instance != null && provider != null) 
            {
                editorService = (IWindowsFormsEditorService)provider.GetService(typeof(IWindowsFormsEditorService));
                this.tempGauge = (TemperatureGauge5) context.Instance;

                if (editorService != null) 
                {
                    // create trackbar control
                    TrackBar tb = new TrackBar();
                    tb.Minimum = 0;
                    tb.Maximum = 100;
                    tb.Orientation = System.Windows.Forms.Orientation.Vertical;
                    tb.TickFrequency = 5;
                    tb.Size = new System.Drawing.Size(75, 160);
                    tb.ValueChanged += new EventHandler(this.ValueChanged);

                    // init with current property value
                    tb.Value = (int)value;

                    editorService.DropDownControl(tb);
                    
                    value = tb.Value;
                }
            }
            return value;
        }

        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) 
        {
            if (context != null && context.Instance != null) 
            {
                return UITypeEditorEditStyle.DropDown;
            }
            
            return base.GetEditStyle(context);
        }

        private void ValueChanged(object sender, EventArgs e) 
        {
            this.tempGauge.Percentage = ((TrackBar)sender).Value;
        }
    }

    [
    DefaultProperty("Percentage"),
    ToolboxBitmap(typeof(TemperatureGauge5), "ToolboxIcon.bmp")
    ]
	public class TemperatureGauge5 : System.Windows.Forms.Control
	{
		private System.ComponentModel.Container components = null;
        private int percent = 50;
        private Color gaugeColor = Color.Red;
        private string fileName = "";

		public TemperatureGauge5()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		private void InitializeComponent()
		{

        }
		#endregion

		protected override void OnPaint(PaintEventArgs pe)
		{
            // compute rects
            Rectangle gaugeRect = new Rectangle( this.Width/6, 0, this.Width-(2*this.Width/6), this.Height-(this.Height/10) );
            Rectangle bulbRect = new Rectangle(0, this.Height-(this.Height/5), this.Width-2, this.Height/5-2);
            Rectangle emptyRect = new Rectangle( gaugeRect.X, gaugeRect.Y, gaugeRect.Width, (gaugeRect.Height/100)*(100-percent));
            Rectangle filledRect = new Rectangle( gaugeRect.X, emptyRect.Y+emptyRect.Height, gaugeRect.Width, gaugeRect.Height-emptyRect.Height );
           
            // draw unfilled rect
            pe.Graphics.DrawRectangle( new Pen(new SolidBrush( gaugeColor )), emptyRect );

            // draw filled portion
            pe.Graphics.FillRectangle( new SolidBrush( gaugeColor ), filledRect );

            // draw bulb
            pe.Graphics.FillEllipse( new SolidBrush( gaugeColor ), bulbRect );
            

            // draw ticks
            for( int i = 1; i < 10; i ++ )
            {
                pe.Graphics.FillEllipse( new SolidBrush( Color.Black ), gaugeRect.X, (gaugeRect.Height/100)*(10*i), gaugeRect.Width/3, 3 );
            }

			base.OnPaint(pe);
		}

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged (e);
            Invalidate();
        }

        [
        Editor(typeof(TemperatureGaugeDesigner), typeof(UITypeEditor)),
        Description("Percentage of gauge that is filled."),Category("Data"),
        ParenthesizePropertyNameAttribute(true), DefaultValue(100)]
        public int Percentage
        {
            get
            {
                return percent;
            }
            set
            {
                if( value < 0 || value > 100 )
                    return;

                percent = value;

                Invalidate();
            }
        }

        // this isn't used. Only here for reference purposes
        [Editor(typeof(System.ComponentModel.Design.CollectionEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public ICollection TemperatureCollection
        {
            get
            {
                return testCollection;
            }
            set
            {
                testCollection = value;
            }
        }

        private ICollection testCollection;

        [Description("Color of filled gauge."),Category("Appearance"), 
        ParenthesizePropertyNameAttribute(true), DefaultValue(typeof(Color), "Red")]
        public Color GaugeColor
        {
            get
            {
                return gaugeColor;
            }
            set
            {
                gaugeColor = value;
                Invalidate();
            }
        }

        [Description("Filename to log runtime data to."),
        Category("Behaviour"), 
        ParenthesizePropertyNameAttribute(true), 
        EditorAttribute(typeof(System.Windows.Forms.Design.FileNameEditor), typeof(System.Drawing.Design.UITypeEditor))
        ]
        public string LogFile
        {
            get
            {
                return fileName;
            }
            set
            {
                fileName = value;
            }
        }
	}
}
