using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace _9_CustomDesigner
{
    [Designer(typeof(RandomShapeDesigner))]
    public class RandomShape : System.Windows.Forms.Control
    {
        private System.ComponentModel.Container components = null;
        private int count = 10;
        ArrayList shapeList;
        private bool initRand = true;

        private enum ShapeType
        {
            Circle = 0,
            FilledCircle = 1,
            Rectangle = 2,
            FilledRectangle = 3
        }

        private struct Shape
        {
            public Point location;
            public Point size;
            public Color color;
            public ShapeType type;
        }

        public RandomShape()
        {
            // This call is required by the Windows.Forms Form Designer.
            InitializeComponent();
            shapeList = new ArrayList( count );
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if( components != null )
                    components.Dispose();
            }
            base.Dispose( disposing );
        }

        #region Component Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            components = new System.ComponentModel.Container();
        }
        #endregion

        protected override void OnPaint(PaintEventArgs pe)
        {
            if( initRand == true )
            {
                Randomize();
                initRand = false;
            }

            foreach( Shape curShape in shapeList )
            {
                switch( curShape.type )
                {
                    case ShapeType.Rectangle :
                    {
                        pe.Graphics.DrawRectangle( new Pen( curShape.color ), curShape.location.X, curShape.location.Y, curShape.size.X, curShape.size.Y );
                        break;
                    }
                    case ShapeType.FilledRectangle:
                    {
                        pe.Graphics.FillRectangle( new SolidBrush(curShape.color), curShape.location.X, curShape.location.Y, curShape.size.X, curShape.size.Y );
                        break;
                    }
                    case ShapeType.Circle:
                    {
                        pe.Graphics.DrawEllipse( new Pen( curShape.color ), curShape.location.X, curShape.location.Y, curShape.size.X, curShape.size.Y );
                        break;
                    }
                    case ShapeType.FilledCircle:
                    {
                        pe.Graphics.FillEllipse( new SolidBrush( curShape.color ), curShape.location.X, curShape.location.Y, curShape.size.X, curShape.size.Y );
                        break;
                    }
                }
            }
        }

        public void Randomize()
        {
            Random randNum = new Random( DateTime.Now.Millisecond );

            shapeList = new ArrayList( this.count );
            for( int i = 0; i < this.count; i++ )
            {
                Shape curShape;
                curShape.location = new Point( randNum.Next(this.Size.Width/2), randNum.Next( this.Size.Height/2 ));
                curShape.size = new Point( randNum.Next(this.Size.Width), randNum.Next(this.Size.Height));
                curShape.color = Color.FromArgb( randNum.Next(255), randNum.Next(255), randNum.Next(255));
                curShape.type = (ShapeType) randNum.Next(4);
                shapeList.Add( curShape );
            }

            Invalidate();
        }

        public int NumShapes
        {
            get
            {
                return this.count;
            }
            set
            {
                this.count = value;
                Randomize();
            }
        }
    }

    public class RandomShapeDesigner : ControlDesigner
    {
        public bool drawBorder = false;

        public override SelectionRules SelectionRules
        {
            get
            {
                return System.Windows.Forms.Design.SelectionRules.TopSizeable | 
                    System.Windows.Forms.Design.SelectionRules.BottomSizeable |
                    System.Windows.Forms.Design.SelectionRules.Moveable |
                    System.Windows.Forms.Design.SelectionRules.Visible;    
            }
        }

        // 9.10 Extending the Design Time Context Menu
        public override System.ComponentModel.Design.DesignerVerbCollection Verbs
        {
            get
            {
                DesignerVerbCollection v = new DesignerVerbCollection();

                v.Add(new DesignerVerb("Randomize", new EventHandler(RandomizeVerbHandler)));
                return v;
            }
        }

        protected override void OnPaintAdornments(PaintEventArgs pe)
        {
            if( drawBorder )
                pe.Graphics.DrawRectangle( new Pen( Color.Black ), 0, 0, this.Control.Width-1, this.Control.Height-1 );
        }

        private void RandomizeVerbHandler(object sender, System.EventArgs e)
        {
            ((RandomShape)this.Control).Randomize();
        }

        private bool DrawDesignBorder
        {
            get
            {
                return this.drawBorder;
            }
            set
            {
                this.drawBorder = value;
                this.Control.Invalidate();
            }
        }

        // 9.11 Removing and Adding Control Properties at Design Time using Metadata Filtering
        protected override void PreFilterProperties(IDictionary properties)
        {
            // call base first
            base.PreFilterProperties (properties);
            properties["DrawDesignBorder"] = TypeDescriptor.CreateProperty( this.GetType(), "DrawDesignBorder", typeof(bool), new Attribute[] {CategoryAttribute.Design});
        }

        // 9.11 Removing Control Properties at Design Time using Metadata Filtering
        protected override void PostFilterProperties(IDictionary properties)
        {
            // remove the Visible property
            properties.Remove("Visible");

            // call base last
            base.PostFilterProperties (properties);
        }

        public bool ShouldSerializeDrawDesignBorder()
        {
            return false;
        }
    }
}
