using System;
using System.IO;

namespace _12_Dir.Net
{
    /// <summary>
    /// Summary description for Class1.
    /// </summary>
    class Class1
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            if( args.Length > 2 )
            {
                DisplayUsage();
                return;
            }

            Dir app;
            if( args.Length <= 1 )
                app = new Dir( Directory.GetCurrentDirectory() );
            else
                app = new Dir( args[0] );

            if( args[1] == "-i" )
            {
                app.ShowDirInfo();
            }
            else if( args[1] == "-e" )
            {
                app.Show();
            }
              
        }

        static void DisplayUsage()
        {
            Console.WriteLine( "Usage: dir.net <directory> <-i|-f> where -i = info and -f = files" );
        }
    }

    public class Dir : IDisposable
    {
        #region IDisposable Members

        public void Dispose()
        {
           swLogFile.Close();
        }

        #endregion

        DirectoryInfo dirInfo = null;
        StreamWriter swLogFile = null;

        public Dir( string path )
        {
            dirInfo = new DirectoryInfo( path );
            CreateLogFile( "temp.log" );
        }

        private void CreateLogFile( string logFile )
        {
            DirectoryInfo di = Directory.CreateDirectory( Directory.GetCurrentDirectory() + "\\temp\\log\\" );
            swLogFile = File.CreateText( di.FullName + logFile );    
        }

        public void ShowDirInfo()
        {
            Console.WriteLine( "Path: {0}", dirInfo.FullName );
            Console.WriteLine( "Root: {0}", dirInfo.Root.ToString() );
            if( dirInfo.Parent != null ) Console.WriteLine( "Parent Dir: {0}", dirInfo.Parent.FullName );
            else Console.WriteLine( "Parent Dir: None" );
            Console.WriteLine( "Attributes: {0}", dirInfo.Attributes.ToString() );
            Console.WriteLine( "Creation Time: {0}", dirInfo.CreationTime.ToString() );
            Console.WriteLine( "Last Access Time: {0}", dirInfo.LastAccessTime.ToString() );
            Console.WriteLine( "Last Write Time: {0}", dirInfo.LastWriteTime.ToString() );

            Console.WriteLine( "Num Subdirectories: {0}", dirInfo.GetDirectories().Length );
            foreach( DirectoryInfo subDir in dirInfo.GetDirectories() )
                Console.WriteLine( "\t{0}", subDir.Name );

            Console.WriteLine( "Num Files: {0}", dirInfo.GetFiles().Length );
            foreach( FileInfo file in dirInfo.GetFiles() )
                Console.WriteLine( "\t{0}", file.Name );

        }

        public void Show()
        {
            Enumerate( dirInfo, 0 );
        }

        private void Enumerate( DirectoryInfo diStart, int indent )
        {
            try 
            {
                foreach (FileSystemInfo diNext in diStart.GetFileSystemInfos()) 
                {
                    for( int i = 0; i < indent; ++i )
                    {
                        swLogFile.Write( "\t" );
                    }

                    if( diNext.GetType() == typeof( DirectoryInfo ))
                    {
                        swLogFile.WriteLine( "< " + diNext.Name + ">");
                        Enumerate( (DirectoryInfo) diNext, ++indent );
                    }
                    else
                    {
                        swLogFile.WriteLine( diNext.Name );
                    }
                }
            }
 
            catch (Exception e) 
            {
                Console.WriteLine( e.Message );
            }
        }
    }
}

