using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace _13_ForeignKeyConstraints
{
    public class Form1 : System.Windows.Forms.Form
    {
        private System.Windows.Forms.DataGrid dgParent;
        private System.Windows.Forms.DataGrid dgChild;
        private System.Windows.Forms.ComboBox cbConstraints;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Button btnSet;
        private DataSet ds;
        private ForeignKeyConstraint fkConstraint;

        private System.ComponentModel.Container components = null;

        public Form1()
        {
            InitializeComponent();
        
            // fill constraint combo
            cbConstraints.Items.Add("Cascade");
            cbConstraints.Items.Add("SetNull");
            cbConstraints.Items.Add("SetDefault");
            cbConstraints.Items.Add("None");
            cbConstraints.SelectedIndex = 3;

            // create tables
            ds = new DataSet();
            DataTable parentTable = ds.Tables.Add( "ParentTable" );
            DataTable childTable = ds.Tables.Add( "ChildTable" );

            // create columns
            parentTable.Columns.Add( "ID", typeof(int) );
            DataColumn linkedParent = parentTable.Columns.Add( "LinkedID", typeof(int) );

            childTable.Columns.Add( "ID", typeof(int) );
            DataColumn linkedChild = childTable.Columns.Add( "LinkedID", typeof(int) );
            
            // create the foreign key constraint and set to None
            fkConstraint = new ForeignKeyConstraint( "LinkedIDFK", linkedParent, linkedChild );
            fkConstraint.UpdateRule = Rule.None;
            fkConstraint.DeleteRule = Rule.None;

            // add constraint to child column
            childTable.Constraints.Add( fkConstraint );

            // data bind the 2 DataGrid's
            dgParent.SetDataBinding( ds, "ParentTable" );
            dgChild.SetDataBinding( ds, "ChildTable" );
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if (components != null) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.dgParent = new System.Windows.Forms.DataGrid();
            this.dgChild = new System.Windows.Forms.DataGrid();
            this.cbConstraints = new System.Windows.Forms.ComboBox();
            this.label1 = new System.Windows.Forms.Label();
            this.btnSet = new System.Windows.Forms.Button();
            ((System.ComponentModel.ISupportInitialize)(this.dgParent)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.dgChild)).BeginInit();
            this.SuspendLayout();
            // 
            // dgParent
            // 
            this.dgParent.DataMember = "";
            this.dgParent.HeaderForeColor = System.Drawing.SystemColors.ControlText;
            this.dgParent.Location = new System.Drawing.Point(8, 8);
            this.dgParent.Name = "dgParent";
            this.dgParent.Size = new System.Drawing.Size(248, 176);
            this.dgParent.TabIndex = 0;
            // 
            // dgChild
            // 
            this.dgChild.DataMember = "";
            this.dgChild.HeaderForeColor = System.Drawing.SystemColors.ControlText;
            this.dgChild.Location = new System.Drawing.Point(8, 192);
            this.dgChild.Name = "dgChild";
            this.dgChild.Size = new System.Drawing.Size(248, 160);
            this.dgChild.TabIndex = 1;
            // 
            // cbConstraints
            // 
            this.cbConstraints.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cbConstraints.Location = new System.Drawing.Point(8, 376);
            this.cbConstraints.Name = "cbConstraints";
            this.cbConstraints.Size = new System.Drawing.Size(168, 21);
            this.cbConstraints.TabIndex = 2;
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(8, 360);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(160, 16);
            this.label1.TabIndex = 3;
            this.label1.Text = "Foreign Key Constraint";
            // 
            // btnSet
            // 
            this.btnSet.Location = new System.Drawing.Point(184, 376);
            this.btnSet.Name = "btnSet";
            this.btnSet.TabIndex = 4;
            this.btnSet.Text = "Set";
            this.btnSet.Click += new System.EventHandler(this.btnSet_Click);
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(264, 422);
            this.Controls.Add(this.btnSet);
            this.Controls.Add(this.label1);
            this.Controls.Add(this.cbConstraints);
            this.Controls.Add(this.dgChild);
            this.Controls.Add(this.dgParent);
            this.Name = "Form1";
            this.Text = "Form1";
            ((System.ComponentModel.ISupportInitialize)(this.dgParent)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.dgChild)).EndInit();
            this.ResumeLayout(false);

        }
        #endregion

        [STAThread]
        static void Main() 
        {
            Application.Run(new Form1());
        }

        private void btnSet_Click(object sender, System.EventArgs e)
        {
            // change the constraint based on ComboBox value
            if( cbConstraints.SelectedItem == null )
                return;

            string constraintName = cbConstraints.Text;
            if( constraintName == "None" )
            {
                fkConstraint.DeleteRule = Rule.None;
                fkConstraint.UpdateRule = Rule.None;
            }
            else if( constraintName == "Cascade" )
            {
                fkConstraint.DeleteRule = Rule.Cascade;
                fkConstraint.UpdateRule = Rule.Cascade;
            }
            else if( constraintName == "SetNull" )
            {
                fkConstraint.DeleteRule = Rule.SetNull;
                fkConstraint.UpdateRule = Rule.SetNull;
            }
            else if (constraintName == "SetDefault" )
            {
                fkConstraint.DeleteRule = Rule.SetDefault;
                fkConstraint.UpdateRule = Rule.SetDefault;
            }
        }
    }
}
