using System;
using System.Data;
using System.Data.SqlClient;

namespace _1_Connection
{
	class Class1
	{
        [STAThread]
        static void Main(string[] args)
        {
            SqlConnection connection;
            SqlDataAdapter dataAdapter;
            DataSet productsDS;
            SqlCommandBuilder commandBuilder;
            SqlCommand selectCommand;
            char input;

            string connectionString = "Integrated Security=SSPI;Data Source=VCSMARKHSCH6;Initial Catalog=Northwind;";
            string selectCmd = "SELECT * From Products";
            
            // create new SqlConnection specifying the connectino string
            connection = new SqlConnection( connectionString );

            // create new dataset object and data adapter used to fill it
            productsDS = new DataSet();
            selectCommand = new SqlCommand( selectCmd, connection );
            dataAdapter = new SqlDataAdapter( selectCommand );     

            // create command builder
            commandBuilder = new SqlCommandBuilder( dataAdapter );

            // open the connection
            connection.Open();

            // fill the dataset from products table
            dataAdapter.Fill(productsDS, "Products" );

            // create event handlers
            dataAdapter.RowUpdated += new SqlRowUpdatedEventHandler(dataAdapter_RowUpdated);
            dataAdapter.RowUpdating +=new SqlRowUpdatingEventHandler(dataAdapter_RowUpdating);
            productsDS.Tables["Products"].RowDeleted +=new DataRowChangeEventHandler(Class1_RowDeleted);
          
            // set primary key
            productsDS.Tables["Products"].PrimaryKey = new DataColumn[]{productsDS.Tables["Products"].Columns["ProductID"]};

            do
            {
                input = ShowMenu();

                switch( input )
                {
                    case( 'V' ):
                    {
                        Console.WriteLine( productsDS.GetXmlSchema() );
                        break;
                    }
                    case( 'H' ):
                    {
                        string customerID;

                        // get customer ID from user
                        Console.Write( "Enter a customer ID: " );

                        customerID = Console.ReadLine();

                        // create new command using SQL stored procedure name
                        SqlCommand myCommand = new SqlCommand("CustOrderHist", connection);

                        // Mark the Command as a stored procedure
                        myCommand.CommandType = CommandType.StoredProcedure;

                        // Add Parameter to SPROC
                        SqlParameter param1 = new SqlParameter("@CustomerID", SqlDbType.NChar, 5);
                        param1.Value = customerID;
                        myCommand.Parameters.Add(param1);

                        SqlDataReader rdr = myCommand.ExecuteReader();
                        if( rdr.FieldCount == 0 )
                            Console.WriteLine( "No orders found for customer" );

                        while( rdr.Read() )
                        {
                            for( int i = 0; i < rdr.FieldCount; i++ )
                                Console.Write( "{0} ", rdr.GetValue(i) );
                            Console.WriteLine();
                        }
                        rdr.Close();

                        break;
                    }
                    case( 'D' ):
                    {
                        SqlCommand select = new SqlCommand(selectCmd, connection );
                        SqlDataReader rdr;
                        rdr = select.ExecuteReader();
                        bool writeColumns = true;

                        while (rdr.Read()) 
                        {
                            if( writeColumns == true )
                            {
                                for( int i = 0; i < rdr.FieldCount; i++ )
                                {
                                    Console.Write(rdr.GetName(i) + "\t" );
                                }
                                Console.WriteLine();
                                writeColumns = false;
                            }
                            for( int i = 0; i < rdr.FieldCount; i++ )
                            {
                                Console.Write( "\"{0}\" ", rdr.GetValue(i).ToString() );
                            }
                            Console.WriteLine();
                        }

                        rdr.Close();

                        break;
                    }
                    case( 'A' ):
                    {
                        // dump INSERT command
                        Console.WriteLine( "INSERT command: {0}\n", commandBuilder.GetInsertCommand().CommandText );
                        Console.WriteLine( "UPDATE command: {0}\n", commandBuilder.GetUpdateCommand().CommandText );
                        Console.WriteLine( "DELETE command: {0}\n", commandBuilder.GetDeleteCommand().CommandText );

                        break;
                    }
                    case( 'Q' ):
                    {
                        connection.Close();

                        break;
                    }
                    
                    default:
                    {
                        Console.WriteLine( "Invalid Command" );
                        break;
                    }
                }
            } while( input != 'Q' );
		}

        static private char ShowMenu()
        {
            Console.WriteLine( "\n\nWhat would you like to do?" );
            Console.WriteLine( "V)iew DataSet schema" );
            Console.WriteLine( "H)istory of Customer Orders" );
            Console.WriteLine( "D)ump Rows" );
            Console.WriteLine( "A)utogenerated SQL Command Dump" );
            Console.WriteLine( "Q)uit" );
            Console.Write( "\nCommand: " );
            return Char.ToUpper(Console.ReadLine()[0]);
        }

        private static void dataAdapter_RowUpdated(object sender, SqlRowUpdatedEventArgs e)
        {
            Console.WriteLine( "*** {0} ***", e.Errors.Message );
        }

        private static void dataAdapter_RowUpdating(object sender, SqlRowUpdatingEventArgs e)
        {
            Console.WriteLine( e.Errors.Message );
        }

        private static void Class1_RowDeleted(object sender, DataRowChangeEventArgs e)
        {
            Console.WriteLine( "** Row Deleted **" );
        }
    }
}
