using System;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace _1_NumberGuessServer
{
    class Class1
    {
        [STAThread]
        static void Main(string[] args)
        {
            Random rand = new Random( DateTime.Now.Millisecond );
            int curRandom;
            try
            {
                TcpListener tcpl = new TcpListener(IPAddress.Any, 2003); // listen on port 2003
                
                tcpl.Start();

                Console.WriteLine("Number Guess Server Started");

                while (true) 
                {
                    string cmd = "";
                    bool bCloseConnection = false;

                    // Accept will block until someone connects
                    Socket s = tcpl.AcceptSocket();
                    
                    SendData( s, "200\r\nOK\r\n\r\n" );

                    PrintClientInfo( s );

                    // generate new random number
                    curRandom = rand.Next( 1, 100 );

                    do
                    {
                        cmd = ReceiveData( s );
                        switch( GetResponseCode( cmd ))
                        {
                            case( 300 ):
                            {
                                // client wants instructions on how to play
                                SendData( s, "200\r\nPick a number between 1 and 100 and the server will return it's too high, too low or correct\r\n\r\n" );
                                break;
                            }
                            case( 301 ):
                            {
                                // client is guessing number
                                try
                                {
                                    int guess = Int32.Parse(GetResponseData( cmd ));
                                    if( guess < curRandom )
                                    {
                                        SendData( s, "401\r\nNumber is too low\r\n\r\n" );
                                    }
                                    else if( guess > curRandom )
                                    {
                                        SendData( s, "401\r\nNumber is too high\r\n\r\n" );
                                    }
                                    else
                                    {
                                        SendData( s, "400\r\nCorrect! The number is " + curRandom + "\r\n\r\n" );
                                        bCloseConnection = true;
                                    }
                                }
                                catch
                                {
                                    SendData( s, "404\r\nInvalid guess!\r\n\r\n" );
                                }

                                break;
                            }
                            case( 302 ):
                            {
                                // client is quitting
                                bCloseConnection = true;
                                break;
                            }
                            default:
                            {
                                break;
                            }
                        }
                    } while( GetResponseCode( cmd ) != 302 && bCloseConnection == false );
                    
                    s.Close();
                }
            }
            catch (SocketException socketError)
            {
                if (socketError.ErrorCode == 10048)
                {
                    Console.WriteLine("Connection to this port failed.  There is another server already running.");
                }
            }
        }

        public static void SendData( Socket client, string response )
        {
            // Convert the string to a Byte Array and send it
            Byte[] data = Encoding.ASCII.GetBytes(response.ToCharArray());
            client.Send(data, data.Length, SocketFlags.None);
        }

        public static string ReceiveData( Socket client )
        {
            string data = "";
            int bytesRecvd = 0;
            int totalBytes = 0;

            byte[] recvData = new byte[1024];   // 1k packet size

            do
            {
                bytesRecvd = client.Receive( recvData, 0, 1024, SocketFlags.None );
                if( bytesRecvd > 0 )
                {
                    data += Encoding.ASCII.GetString( recvData );
                    totalBytes += bytesRecvd;
                }
            } while( bytesRecvd == 1024 );
            
            return data;
        }

        // methods to break apart protocol messages
        // extracts the numerical identifier at beginning of message
        public static int GetResponseCode( string response )
        {
            string code = response.Substring( 0, response.IndexOf( '\r' ));
            return Int32.Parse( code );
        }

        // extracts extra message data
        public static string GetResponseData( string response )
        {
            return response.Substring( response.IndexOf('\n')+1, response.LastIndexOf("\r\n\r\n")-response.IndexOf('\n')+1 );
        }

        public static void PrintClientInfo( Socket s )
        {
            Console.WriteLine( "Client End Point: IP: {0} Port: {1}", 
                ((IPEndPoint)s.RemoteEndPoint).Address.ToString(), 
                ((IPEndPoint)s.RemoteEndPoint).Port.ToString() );

            Console.WriteLine( "Server End Point: IP: {0} Port: {1}", 
                ((IPEndPoint)s.LocalEndPoint).Address.ToString(),
                ((IPEndPoint)s.LocalEndPoint).Port.ToString() );
            
            Console.WriteLine( "Address Family: {0}", s.AddressFamily.ToString() );
            Console.WriteLine( "Currently connected: {0}", s.Connected.ToString() );
            Console.WriteLine( "Protocol Type: {0}", s.ProtocolType.ToString() );
            Console.WriteLine( "Socket Type: {0}", s.SocketType.ToString() );
            Console.WriteLine( "Server support IPv6: {0}", Socket.SupportsIPv6.ToString() );
        }
    }
}
