using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;

namespace _5_NumberGuessClient
{
    /// <summary>
    /// Summary description for Class1.
    /// </summary>
    class Class1
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            TcpClient client = new TcpClient();
            Byte[] read = new Byte[1024];

            string gameServer = "";

            if (args.Length != 1) 
            {
                gameServer = "127.0.0.1";
            }
            else
            {
                gameServer = args[0];
            }
            
            // Verify that the server exists
            IPHostEntry serverIP = Dns.Resolve( gameServer );
            if( serverIP == null )
            {
                Console.WriteLine("Cannot find server: {0}", gameServer);
                return;
            }
            else
            {
                Console.WriteLine( "Found server. {0}", serverIP.HostName );
            }

            // Try to connect to the game server on port 2003
            try 
            {
                client.Connect(gameServer, 2003);
            } 
            catch (SocketException e) 
            {
                Console.WriteLine("Cannot connect to {0}: {1}", gameServer, e.Message);
                return;
            }

            // Get the stream
            Stream s;
            try 
            {
                s = client.GetStream();
            } 
            catch (InvalidOperationException exc) 
            {
                Console.WriteLine("Cannot connect to {0}: {1}", gameServer, exc.Message);
                return;
            }

            Console.WriteLine( "Connected to game server" );
            string response = GetResponse(s);
            Console.WriteLine( "Response\r\n--------\r\nCode: {0} Message: {1} ", 
                GetResponseCode(response), GetResponseData(response));

            // connected begin game
            StartGame( s );

            client.Close();

            // Wait for user response to exit
            Console.WriteLine("Press Return to exit");
            Console.Read();

            client.Close();
        }

        public static void StartGame( Stream s )
        {
            string cmd;

            do
            {
                Console.WriteLine( "\nSelect a menu item or try to guess the number: ");
                Console.WriteLine( "I)nstructions" );
                Console.WriteLine( "Q)uit" );
                Console.Write( "Command:> " );

                cmd = Console.ReadLine();
                cmd = cmd.ToUpper();

                switch( cmd )
                {
                    case( "I" ):
                    {
                        SendData( s, "300\r\n\r\n" );
                        Console.WriteLine( "Instructions: {0}", GetResponseData( GetResponse( s )));
                        break;
                    }
                    case( "Q" ):
                    {
                        SendData( s, "302\r\n\r\n" );
                        break;
                    }
                    default:
                    {
                        try
                        {
                            int guess = Int32.Parse( cmd );

                            SendData( s, "301" + "\r\n" + guess.ToString() + "\r\n\r\n");
                            string response = GetResponse( s );
                            Console.WriteLine( "Server says: {0}", GetResponseData( response));
                            if( GetResponseCode( response ) == 400 )
                            {
                                // force a game over
                                cmd = "Q";
                            }
                        }
                        catch
                        {
                            Console.WriteLine( "Invalid command. Try again. " );
                        }
                        break;
                    }
                }

            } while( cmd != "Q" );

            Console.WriteLine( "Goodbye" );
        }

        public static string GetResponse( Stream server )
        {
            string response = "";
            byte[] buffer = new byte[32];

            int bytes = 0;
            int totalBytes = 0;

            while( (bytes = server.Read( buffer, 0, buffer.Length )) == buffer.Length )
            {
                response += Encoding.ASCII.GetString(buffer);
                totalBytes += bytes;
            }

            if( bytes > 0  && bytes < 32 )
            {
                response += Encoding.ASCII.GetString(buffer);
                totalBytes += bytes;
            }

            return response;
        }

        public static void SendData( Stream server, string data )
        {
            Byte[] byteData = Encoding.ASCII.GetBytes(data.ToCharArray());
            server.Write( byteData, 0, byteData.Length );

            return;
        }

        public static int GetResponseCode( string response )
        {
            string code = response.Substring( 0, response.IndexOf( '\r' ));
            return Int32.Parse( code );
        }

        public static string GetResponseData( string response )
        {
            return response.Substring( response.IndexOf('\n')+1, response.LastIndexOf("\r\n\r\n")-response.IndexOf('\n')+1 );
        }
    }
}
