using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Collections;

namespace _5_UDPServer
{
    class Class1
    {
        [STAThread]
        static void Main(string[] args)
        {
            UDPServer server = new UDPServer();
            server.Run();
        }
    }

    struct ConnectedClient
    {
        public IPEndPoint ip;
        public string name;
    }

    class UDPServer
    {
        private UdpClient udp;
        private IPEndPoint remoteEndPoint;
        private ArrayList clients;

        public UDPServer()
        {
            clients = new ArrayList();
            udp = new UdpClient( 2003 );
        }

        public void Run()
        {
            while( true )
            {
                remoteEndPoint = new IPEndPoint( new IPAddress(0), 0 );

                byte[] data = udp.Receive( ref remoteEndPoint );

                Console.WriteLine( "Received data: {0}", Encoding.ASCII.GetString( data ));

                ProcessMessage( Encoding.ASCII.GetString( data ));
            }
        }

        private void ProcessMessage( string message )
        {
            switch( GetResponseCode( message ) )
            {
                case (300): // add new user
                {
                    ConnectedClient newClient;
                    newClient.ip = remoteEndPoint;
                    newClient.name = GetResponseData( message );

                    clients.Add( newClient );
                    BroadcastMessage( newClient.name + " has entered the conversation." );
                    break;
                }
                case( 400 ): // chat
                {

                    BroadcastMessage( FindName( remoteEndPoint ) + ": " + GetResponseData( message ) );
                    break;
                }
                case (301):
                {
                    BroadcastMessage( FindName( remoteEndPoint) + " has left the conversation." );
                    RemoveClient( remoteEndPoint );
                    break;
                }
            }
        }

        private void RemoveClient( IPEndPoint endPoint )
        {
            foreach( object obj in clients )
            {
                ConnectedClient client = (ConnectedClient) obj;
                if( client.ip.Address.Equals(endPoint.Address) && client.ip.Port == endPoint.Port )
                {
                    clients.Remove( obj );
                    return;
                }
            }
        }

        private string FindName( IPEndPoint endPoint )
        {
            foreach( object obj in clients )
            {
                ConnectedClient client = (ConnectedClient) obj;
                if( client.ip.Address.Equals(endPoint.Address) && client.ip.Port == endPoint.Port )
                    return client.name;
            }

            return "???";
        }

        private void BroadcastMessage( string message )
        {
            foreach( object obj in clients )
            {
                try
                {
                    ConnectedClient client = (ConnectedClient) obj;
                    udp.Send( Encoding.ASCII.GetBytes( message.ToCharArray ()), message.Length, client.ip ); 
                }
                catch( Exception e)
                {
                    Console.WriteLine( e.Message );
                }
            }
        }
        
        // methods to break apart protocol messages
        // extracts the numerical identifier at beginning of message
        public static int GetResponseCode( string response )
        {
            string code = response.Substring( 0, response.IndexOf( ' ' ));
            return Int32.Parse( code );
        }

        // extracts extra message data
        public static string GetResponseData( string response )
        {
            int space = response.IndexOf( ' ' ) + 1;
            
            return response.Substring( space, response.Length-space );
        }
    }
}