using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;

namespace _1_LotteryService
{
    [ WebService( Namespace="http://samspublishing/1_LotteryService",
          Description="Provides the ability to generate a set amount of random numbers",
          Name="Lottery Web Service")]
    public class LotteryNumberGenerator : System.Web.Services.WebService
    {
        public LotteryNumberGenerator()
        {
            //CODEGEN: This call is required by the ASP.NET Web Services Designer
            InitializeComponent();
        }

        #region Component Designer generated code
		
        //Required by the Web Services Designer 
        private IContainer components = null;
				
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if(disposing && components != null)
            {
                components.Dispose();
            }
            base.Dispose(disposing);		
        }
		
		#endregion

        [WebMethod( Description="Generates a list of numbers between Min and Max with or without duplicates" )]
        public int[] GenerateNumbers(int Count, int Min, int Max, bool AllowDuplicates)
        {
            Random rand = new Random( DateTime.Now.Millisecond );
            ArrayList numbers = new ArrayList( Count );

            for( int i = 0; i < Count; i++ )
            {
                int newNum = rand.Next( Min, Max );
                if( AllowDuplicates == false )
                {
                    while( numbers.Contains( newNum ) == true ) newNum = rand.Next( Min, Max );
                }

                numbers.Add( newNum );
            }
            
            return (int[]) numbers.ToArray(typeof(int));
        }

        [WebMethod( Description="Begins state enabled random number generation",
             EnableSession=true)]
        public void BeginNumberList( int Min, int Max, bool AllowDuplicates )
        {
            Session["Min"] = Min;
            Session["Max"] = Max;
            Session["Duplicates"] = AllowDuplicates;
            Session["NumberList"] = new ArrayList();
        }

        [WebMethod( Description="Gets next number for state enabled number generation.",
             EnableSession=true)]
        public int NextNumber()
        {
            if( Session["Min"] == null || Session["Max"] == null || 
                Session["Duplicates"] == null || Session["NumberList"] == null )
                return -1;
                                                                                  
            // generate random number
            Random rand = new Random( DateTime.Now.Millisecond );
            int newNum = rand.Next( (int) Session["Min"], (int) Session["Max"] );

            // check if duplicates are allowed
            if( ((bool)Session["Duplicates"]) == false )
            {
                // get current list and check if any numbers still available
                ArrayList list = (ArrayList) Session["NumberList"];;
                if( list.Count >= (int)Session["Max"] - (int)Session["Min"] ) return -1;

                // generate a non-duplicate random number
                while( list.Contains( newNum ) == true ) newNum = rand.Next( (int)Session["Min"], (int)Session["Max"] );

                // save in list
                list.Add( newNum );
            }

            return newNum;
        }

        [WebMethod( Description="Ends state enabled random number generation",
             EnableSession=true)]
        public void EndNumberList()
        {
            Session.Remove( "Min" );
            Session.Remove( "Max" );
            Session.Remove( "Duplicates" );
            Session.Remove( "NumberList" );
        }

        [WebMethod( Description="Generates a random number" )]
        public int GenerateNumber()
        {
            Random rand = new Random( DateTime.Now.Millisecond );
            return rand.Next();
        }

        [WebMethod( Description="Generates a random number within a range", MessageName="GenerateNumberInRange" )]
        public int GenerateNumber( int Min, int Max )
        {
            Random rand = new Random( DateTime.Now.Millisecond );
            return rand.Next( Min, Max );
        }
	}
}
