using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Reflection;
using _11_NotepadPluginSDK;

namespace _11_NotepadWithPlugins
{
    public class Form1 : System.Windows.Forms.Form
    {
        private System.Windows.Forms.MainMenu mainMenu1;
        private System.Windows.Forms.MenuItem menuItem1;
        private System.Windows.Forms.MenuItem mnuNew;
        private System.Windows.Forms.MenuItem mnuOpen;
        private System.Windows.Forms.MenuItem mnuSave;
        private System.Windows.Forms.MenuItem mnuSaveAs;
        private System.Windows.Forms.MenuItem mnuExit;
        private System.Windows.Forms.MenuItem mnuPlugins;
        private System.Windows.Forms.MenuItem menuItem8;
        private System.Windows.Forms.TextBox tbContent;
        private System.Windows.Forms.OpenFileDialog openFileDialog1;
        private System.Windows.Forms.SaveFileDialog saveFileDialog1;
        private System.ComponentModel.Container components = null;
        private string fileName = null;
        private ArrayList plugins;

        public Form1()
        {
            InitializeComponent();

            plugins = new ArrayList();

            // enumerate files in plugin directory
            foreach( string fileName in Directory.GetFiles( Environment.CurrentDirectory + @"\plugins", "*.dll" ))
            {
                // load assembly
                Assembly asm = Assembly.LoadFile( fileName );

                // get all modules in assembly
                foreach( Module module in asm.GetModules(false) )
                {
                    // for all types
                    foreach( Type t in module.GetTypes() )
                    {
                        // check to see if it supports the INotepadPlugin interface
                        foreach (Type iface in t.FindInterfaces( new TypeFilter(PluginInterfaceFilter) , "INotepadPlugin" ))
                        {
                            INotepadPlugin plugin = (INotepadPlugin)Activator.CreateInstance(t);
                            plugins.Add(plugin);
                            mnuPlugins.MenuItems.Add( plugin.Name, new EventHandler(mnuPlugin_Click) );
                        }
                    }
                }      
            }
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if (components != null) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.tbContent = new System.Windows.Forms.TextBox();
            this.mainMenu1 = new System.Windows.Forms.MainMenu();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.mnuNew = new System.Windows.Forms.MenuItem();
            this.mnuOpen = new System.Windows.Forms.MenuItem();
            this.mnuSave = new System.Windows.Forms.MenuItem();
            this.mnuSaveAs = new System.Windows.Forms.MenuItem();
            this.menuItem8 = new System.Windows.Forms.MenuItem();
            this.mnuExit = new System.Windows.Forms.MenuItem();
            this.mnuPlugins = new System.Windows.Forms.MenuItem();
            this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
            this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
            this.SuspendLayout();
            // 
            // tbContent
            // 
            this.tbContent.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tbContent.Location = new System.Drawing.Point(0, 0);
            this.tbContent.Multiline = true;
            this.tbContent.Name = "tbContent";
            this.tbContent.Size = new System.Drawing.Size(568, 266);
            this.tbContent.TabIndex = 0;
            this.tbContent.Text = "";
            // 
            // mainMenu1
            // 
            this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                      this.menuItem1,
                                                                                      this.mnuPlugins});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                      this.mnuNew,
                                                                                      this.mnuOpen,
                                                                                      this.mnuSave,
                                                                                      this.mnuSaveAs,
                                                                                      this.menuItem8,
                                                                                      this.mnuExit});
            this.menuItem1.Text = "&File";
            // 
            // mnuNew
            // 
            this.mnuNew.Index = 0;
            this.mnuNew.Text = "&New";
            this.mnuNew.Click += new System.EventHandler(this.mnuNew_Click);
            // 
            // mnuOpen
            // 
            this.mnuOpen.Index = 1;
            this.mnuOpen.Text = "&Open";
            this.mnuOpen.Click += new System.EventHandler(this.mnuOpen_Click);
            // 
            // mnuSave
            // 
            this.mnuSave.Index = 2;
            this.mnuSave.Text = "&Save";
            this.mnuSave.Click += new System.EventHandler(this.mnuSave_Click);
            // 
            // mnuSaveAs
            // 
            this.mnuSaveAs.Index = 3;
            this.mnuSaveAs.Text = "&Save As";
            this.mnuSaveAs.Click += new System.EventHandler(this.mnuSaveAs_Click);
            // 
            // menuItem8
            // 
            this.menuItem8.Index = 4;
            this.menuItem8.Text = "-";
            // 
            // mnuExit
            // 
            this.mnuExit.Index = 5;
            this.mnuExit.Text = "E&xit";
            this.mnuExit.Click += new System.EventHandler(this.mnuExit_Click);
            // 
            // mnuPlugins
            // 
            this.mnuPlugins.Index = 1;
            this.mnuPlugins.Text = "&Plugins";
            // 
            // openFileDialog1
            // 
            this.openFileDialog1.Filter = "Text Files (*.txt)|*.txt";
            // 
            // saveFileDialog1
            // 
            this.saveFileDialog1.Filter = "Text Files (*.txt)|*.txt";
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(568, 266);
            this.Controls.Add(this.tbContent);
            this.Menu = this.mainMenu1;
            this.Name = "Form1";
            this.Text = "Notepad (with Plugins)";
            this.ResumeLayout(false);

        }
        #endregion

        [STAThread]
        static void Main() 
        {
            Application.Run(new Form1());
        }

        public static bool PluginInterfaceFilter( Type typeObj, object criteria )
        {
            if( typeObj.Name == (string) criteria )
                return true;
            return false;
        }

        private void mnuPlugin_Click(object sender, System.EventArgs e)
        {
            MenuItem item = (MenuItem) sender;
            INotepadPlugin plugin = (INotepadPlugin) plugins[item.Index];
            
            if( tbContent.SelectionLength > 0 )
                tbContent.SelectedText = plugin.Execute( tbContent.SelectedText );
            else
                tbContent.Text = plugin.Execute( tbContent.Text );
        }

        private void mnuNew_Click(object sender, System.EventArgs e)
        {
            tbContent.Text = "";
            fileName = null;
        }

        private void mnuOpen_Click(object sender, System.EventArgs e)
        {
            if( openFileDialog1.ShowDialog() == DialogResult.Cancel )
                return;

            tbContent.Text = "";
            fileName = openFileDialog1.FileName;
            OpenText( fileName );
        }

        private void mnuSave_Click(object sender, System.EventArgs e)
        {
            if( fileName == null )
            {
                mnuSaveAs_Click( sender, e );
                return;
            }

            SaveText( fileName );
        }

        private void mnuSaveAs_Click(object sender, System.EventArgs e)
        {
            if( saveFileDialog1.ShowDialog() == DialogResult.OK )
            {
                SaveText( saveFileDialog1.FileName );
                fileName = saveFileDialog1.FileName;
            }        
        }

        private void mnuExit_Click(object sender, System.EventArgs e)
        {
            this.Close();
        }

        private void OpenText( string fileName )
        {
            StreamReader sr = File.OpenText(fileName);
            tbContent.Text = sr.ReadToEnd();
            sr.Close();
        }

        private void SaveText( string fileName )
        {
            StreamWriter sw = File.CreateText(fileName);
            sw.Write( tbContent.Text);
            sw.Close();
        }
    }
}
