using System;
using System.Reflection;
using System.Reflection.Emit;
using System.Threading;

namespace _9_DynamicAssembly
{
    class Class1
    {
        [STAThread]
        static void Main(string[] args)
        {
            string input;
            AnimalBuilder builder = new AnimalBuilder();

            Console.WriteLine( "Welcome to the .NET Animal Builder!" );
            Console.WriteLine( "This program lets you build an animal and save it as an assembly" );

            input = GetInput( "What type of animal do you want?" );
            builder.AnimalType = input;
            
            input = GetInput( "What sound does the animal make?" );
            builder.AnimalSound = input;

            input = GetInput( "How many sounds will it make before it gets hoarse (no pun intented)?" );
            builder.SoundsBeforeHoarse = Int32.Parse(input);

            input = GetInput( "Where would you like to save this animal?" );
            builder.BuildAndSaveAnimal( input );
        }

        static string GetInput( string question )
        {
            Console.Write( question + ": " );
            return Console.ReadLine();
        }

        public class AnimalBuilder
        {
            private string animalType;
            private string animalSound;
            private int sounds;

            public string AnimalType
            {
                get
                {
                    return animalType;
                }
                set
                {
                    animalType = value;
                }
            }

            public string AnimalSound
            {
                get
                {
                    return animalSound;
                }
                set
                {
                    animalSound = value;
                }
            }

            public int SoundsBeforeHoarse
            {
                get
                {
                    return sounds;
                }
                set
                {
                    sounds = value;
                }
            }

            public void BuildAndSaveAnimal( string assemblyPath )
            {
                // create an assembly name
                AssemblyName asmName = new AssemblyName();
                asmName.Name = animalType;

                // get current appdomain
                AppDomain domain = Thread.GetDomain();

                // get an assembly builder from the appdomain
                AssemblyBuilder asmBuilder = domain.DefineDynamicAssembly( asmName, AssemblyBuilderAccess.Save );

                // get a module builder from the assembly builder
                ModuleBuilder modBuilder = asmBuilder.DefineDynamicModule( animalType, assemblyPath );

                // create a new public class
                TypeBuilder typeBuilder = modBuilder.DefineType( animalType, TypeAttributes.Public );

                // add a new field
                FieldBuilder soundFld = typeBuilder.DefineField( "Sound", typeof(string), FieldAttributes.Private );
                FieldBuilder countFld = typeBuilder.DefineField( "Count", typeof(int), FieldAttributes.Private );
                
                // add the hoarse method
                MethodBuilder hoarseMethod = typeBuilder.DefineMethod( "MakeHoarse", MethodAttributes.Public, null, new Type[]{} );
                ILGenerator gen = hoarseMethod.GetILGenerator();

                // create labels for branching
                Label endLoop = gen.DefineLabel();
                Label beginLoop = gen.DefineLabel();

                // create Console.WriteLine MethodInfo
                MethodInfo writeLine = typeof(Console).GetMethod( "WriteLine", new Type[]{typeof(string)});
               
                gen.DeclareLocal( typeof(int) );
                gen.Emit( OpCodes.Ldc_I4_0 );
                gen.Emit( OpCodes.Stloc_0 );
                gen.Emit( OpCodes.Br_S, endLoop );
                gen.MarkLabel( beginLoop );
                gen.Emit( OpCodes.Ldarg_0 );
                gen.Emit( OpCodes.Ldfld, soundFld );
                gen.EmitCall( OpCodes.Call, writeLine, null );
                gen.Emit( OpCodes.Ldloc_0 );
                gen.Emit( OpCodes.Ldc_I4_1 );
                gen.Emit( OpCodes.Add );
                gen.Emit( OpCodes.Stloc_0 );
                gen.MarkLabel( endLoop );
                gen.Emit( OpCodes.Ldloc_0 );
                gen.Emit( OpCodes.Ldarg_0 );
                gen.Emit( OpCodes.Ldfld, countFld );
                gen.Emit( OpCodes.Blt_S, beginLoop );
                gen.EmitWriteLine("...a muffled sound emanates..." );
                gen.Emit( OpCodes.Ret );
                
                // build the constructor
                ConstructorBuilder constructor = typeBuilder.DefineConstructor( MethodAttributes.Public, CallingConventions.Standard, null );
                gen = constructor.GetILGenerator();

                gen.Emit( OpCodes.Ldarg_0 );
                gen.Emit( OpCodes.Call, typeof(Object).GetConstructor(new Type[]{}));
                gen.Emit( OpCodes.Ldarg_0 );
                gen.Emit( OpCodes.Ldstr, animalSound );
                gen.Emit( OpCodes.Stfld, soundFld );
                gen.Emit( OpCodes.Ldarg_0 );
                gen.Emit( OpCodes.Ldc_I4, sounds );
                gen.Emit( OpCodes.Stfld, countFld );
                gen.Emit( OpCodes.Ret );

                typeBuilder.CreateType();

                // save final assembly
                asmBuilder.Save( assemblyPath );
            }

            // Constructor IL Code
            /*
            IL_0000:  ldarg.0
            IL_0001:  call       instance void [mscorlib]System.Object::.ctor()
            IL_0006:  ldarg.0
            IL_0007:  ldstr      "moo"
            IL_000c:  stfld      string ConsoleApplication1.Horse::Sound
            IL_0011:  ldarg.0
            IL_0012:  ldc.i4.5
            IL_0013:  stfld      int32 ConsoleApplication1.Horse::Count
            IL_0018:  ret
            */
            // MakeHoarse IL Code
            /*
                IL_0000:  ldc.i4.0
                IL_0001:  stloc.0
                IL_0002:  br.s       IL_0013
                IL_0004:  ldarg.0
                IL_0005:  ldfld      string ConsoleApplication1.Horse::Sound
                IL_000a:  call       void [mscorlib]System.Console::WriteLine(string)
                IL_000f:  ldloc.0
                IL_0010:  ldc.i4.1
                IL_0011:  add
                IL_0012:  stloc.0
                IL_0013:  ldloc.0
                IL_0014:  ldarg.0
                IL_0015:  ldfld      int32 ConsoleApplication1.Horse::Count
                IL_001a:  blt.s      IL_0004
                IL_001c:  ldstr      "...a muffled sound emanates from the animal."
                IL_0021:  call       void [mscorlib]System.Console::WriteLine(string)
                IL_0026:  ret
                */
        }
	}
}
