using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;

namespace _7_IRChatClient
{
    public class Form1 : System.Windows.Forms.Form
    {
        private System.Windows.Forms.TextBox tbTranscript;
        private System.Windows.Forms.TextBox tbMessage;
        private System.Windows.Forms.Button btnSend;
        private System.Windows.Forms.MainMenu mainMenu1;
        private System.Windows.Forms.MenuItem menuItem1;
        private System.Windows.Forms.MenuItem mnuExit;
        private Thread listenerThread;
        private string serviceType = "IRDA_CHAT";
        private System.Windows.Forms.MenuItem mnuDevInfo;
        IrDAListener listener; 

        public Form1()
        {
            InitializeComponent();
            listener = new IrDAListener( serviceType );
            listenerThread = new Thread( new ThreadStart( ListenerThreadMethod ));
            listenerThread.Start();
        }

        void ListenerThreadMethod()
        {
            int bytesRead = 0;

            IrDAClient client = null;   // the connecting client

            Stream byteStream = null;
            byte[] buffer = new byte[1024];
            
            try
            {
                do
                {
                    listener.Start();

                    client = listener.AcceptIrDAClient();
                    byteStream = client.GetStream();
                    bytesRead = byteStream.Read( buffer, 0, 1024 );

                    if( byteStream != null )
                        byteStream.Close();

                    if( client != null )
                        client.Close();

                    AddMessage( client.RemoteMachineName, Encoding.ASCII.GetString( buffer, 0, bytesRead ));
                } while (true);
            }
            finally
            {
                if( byteStream != null )
                    byteStream.Close();

                if( client != null )
                    client.Close();

                listener.Stop();
            }
        }

		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.mainMenu1 = new System.Windows.Forms.MainMenu();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.mnuExit = new System.Windows.Forms.MenuItem();
            this.tbTranscript = new System.Windows.Forms.TextBox();
            this.tbMessage = new System.Windows.Forms.TextBox();
            this.btnSend = new System.Windows.Forms.Button();
            this.mnuDevInfo = new System.Windows.Forms.MenuItem();
            // 
            // mainMenu1
            // 
            this.mainMenu1.MenuItems.Add(this.menuItem1);
            // 
            // menuItem1
            // 
            this.menuItem1.MenuItems.Add(this.mnuDevInfo);
            this.menuItem1.MenuItems.Add(this.mnuExit);
            this.menuItem1.Text = "&File";
            // 
            // mnuExit
            // 
            this.mnuExit.Text = "E&xit";
            this.mnuExit.Click += new System.EventHandler(this.mnuExit_Click);
            // 
            // tbTranscript
            // 
            this.tbTranscript.Location = new System.Drawing.Point(8, 8);
            this.tbTranscript.Multiline = true;
            this.tbTranscript.ReadOnly = true;
            this.tbTranscript.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.tbTranscript.Size = new System.Drawing.Size(224, 208);
            this.tbTranscript.Text = "";
            // 
            // tbMessage
            // 
            this.tbMessage.Location = new System.Drawing.Point(8, 224);
            this.tbMessage.Size = new System.Drawing.Size(168, 22);
            this.tbMessage.Text = "";
            // 
            // btnSend
            // 
            this.btnSend.Location = new System.Drawing.Point(184, 224);
            this.btnSend.Size = new System.Drawing.Size(48, 20);
            this.btnSend.Text = "Send";
            this.btnSend.Click += new System.EventHandler(this.btnSend_Click);
            // 
            // mnuDevInfo
            // 
            this.mnuDevInfo.Text = "GetDevInfo";
            this.mnuDevInfo.Click += new System.EventHandler(this.mnuDevInfo_Click);
            // 
            // Form1
            // 
            this.Controls.Add(this.btnSend);
            this.Controls.Add(this.tbMessage);
            this.Controls.Add(this.tbTranscript);
            this.Menu = this.mainMenu1;
            this.Text = "IRChat";

        }
		#endregion

		static void Main() 
		{
			Application.Run(new Form1());
            
		}

        private void mnuExit_Click(object sender, System.EventArgs e)
        {
            listener.Stop();
            this.Close();
        }

        private void AddMessage( string name, string message )
        {
            tbTranscript.Text += name + ": " + message + "\r\n";
            tbTranscript.ScrollToCaret();
        }

        private void btnSend_Click(object sender, System.EventArgs e)
        {
            int numTries = 0;
            const int numRetries = 5;
            IrDAClient client = null;   // the client to send data to
           
            // disable the send button and display the wait cursor
            btnSend.Enabled = false;
            Cursor.Current = Cursors.WaitCursor;

            do
            {
                try
                {
                    client = new IrDAClient( serviceType );
                    IrDADeviceInfo[] devices = client.DiscoverDevices(5);
                    foreach( IrDADeviceInfo device in devices )
                    {
                        AddMessage( "Found DeviceID", Encoding.ASCII.GetString(device.DeviceID,0,device.DeviceID.Length) );
                        AddMessage( "Found DeviceName", device.DeviceName );
                        AddMessage( "Found Hints", device.Hints.ToString() );
                    }
                }
                catch( Exception ex )
                {
                    if( numTries >= numRetries-1 )
                    {
                        AddMessage( "Error", "cannot send message-" + ex.Message );
                        btnSend.Enabled = true;
                        tbMessage.Text = "";
                        Cursor.Current = Cursors.Default;
                        return;
                    }
                }

                numTries++;

            } while( (client == null) && (numTries < numRetries ));

            Stream byteStream = null;
            try
            {
                byteStream = client.GetStream();
                byteStream.Write( Encoding.ASCII.GetBytes( tbMessage.Text ), 0, tbMessage.Text.Length );
                AddMessage( "You", tbMessage.Text );
            }
            finally
            {
                if( byteStream != null )
                    byteStream.Close();

                if( client != null )
                    client.Close();

                btnSend.Enabled = true;
                tbMessage.Text = "";
                Cursor.Current = Cursors.Default;
            }

        }

        private void mnuDevInfo_Click(object sender, System.EventArgs e)
        {
            IrDAClient client = new IrDAClient();
            IrDADeviceInfo[] devices = client.DiscoverDevices(10);
            foreach( IrDADeviceInfo device in devices )
            {
                AddMessage( "Found", "DeviceID = " + FormatDeviceID(device.DeviceID) + " DeviceName: " + device.DeviceName + " Hints: " + device.Hints.ToString() );
            }
        }

        private string FormatDeviceID( byte[] deviceID )
        {
            string ret = "0x";
            
            for( int i = 0; i < 4; i++ )
            {
                ret += Convert.ToInt32(deviceID[i]).ToString("X");
            }

            return ret;
        }
	}
}
